"""
import all wrapper function
"""

# import mouse
from je_auto_control.wrapper.auto_control_mouse import click_mouse
from je_auto_control.wrapper.auto_control_mouse import mouse_table
from je_auto_control.wrapper.auto_control_mouse import position
from je_auto_control.wrapper.auto_control_mouse import press_mouse
from je_auto_control.wrapper.auto_control_mouse import release_mouse
from je_auto_control.wrapper.auto_control_mouse import scroll
from je_auto_control.wrapper.auto_control_mouse import set_position
from je_auto_control.wrapper.auto_control_mouse import special_table

# import keyboard
from je_auto_control.wrapper.auto_control_keyboard import keys_table
from je_auto_control.wrapper.auto_control_keyboard import press_key
from je_auto_control.wrapper.auto_control_keyboard import release_key
from je_auto_control.wrapper.auto_control_keyboard import type_key
from je_auto_control.wrapper.auto_control_keyboard import check_key_is_press
from je_auto_control.wrapper.auto_control_keyboard import write
from je_auto_control.wrapper.auto_control_keyboard import hotkey

# import screen
from je_auto_control.wrapper.auto_control_screen import size
from je_auto_control.wrapper.auto_control_screen import screenshot

# import image
from je_auto_control.wrapper.auto_control_image import locate_all_image
from je_auto_control.wrapper.auto_control_image import locate_image_center
from je_auto_control.wrapper.auto_control_image import locate_and_click

# Critical
from je_auto_control.utils.critical_exit.critcal_exit import CriticalExit

# Exception
from je_auto_control.utils.je_auto_control_exception.exceptions import AutoControlException
from je_auto_control.utils.je_auto_control_exception.exceptions import AutoControlKeyboardException
from je_auto_control.utils.je_auto_control_exception.exceptions import AutoControlMouseException
from je_auto_control.utils.je_auto_control_exception.exceptions import AutoControlCantFindKeyException
from je_auto_control.utils.je_auto_control_exception.exceptions import AutoControlScreenException
from je_auto_control.utils.je_auto_control_exception.exceptions import ImageNotFoundException
from je_auto_control.utils.je_auto_control_exception.exceptions import AutoControlJsonActionException
from je_auto_control.utils.je_auto_control_exception.exceptions import AutoControlRecordException
from je_auto_control.utils.je_auto_control_exception.exceptions import AutoControlActionNullException
from je_auto_control.utils.je_auto_control_exception.exceptions import AutoControlActionException

# record
from je_auto_control.wrapper.auto_control_record import record
from je_auto_control.wrapper.auto_control_record import stop_record

# json
from je_auto_control.utils.action_executer.action_executor import execute_action
from je_auto_control.utils.action_file.json_file import read_action_json
from je_auto_control.utils.action_file.json_file import write_action_json



from je_auto_control import AutoControlActionException
from je_auto_control import AutoControlActionNullException
from je_auto_control import check_key_is_press
from je_auto_control import click_mouse
from je_auto_control import hotkey
from je_auto_control import keys_table
from je_auto_control import locate_all_image
from je_auto_control import locate_and_click
from je_auto_control import locate_image_center
from je_auto_control import mouse_table
from je_auto_control import position
from je_auto_control import press_key
from je_auto_control import press_mouse
from je_auto_control import release_key
from je_auto_control import release_mouse
from je_auto_control import screenshot
from je_auto_control import scroll
from je_auto_control import set_position
from je_auto_control import size
from je_auto_control import special_table
from je_auto_control import type_key
from je_auto_control import write
from je_auto_control.utils.je_auto_control_exception.exception_tag import action_is_null_error
from je_auto_control.utils.je_auto_control_exception.exception_tag import cant_execute_action_error

event_dict = {
    # mouse
    "mouse_left": ("click_mouse", click_mouse),
    "mouse_right": ("click_mouse", click_mouse),
    "mouse_middle": ("click_mouse", click_mouse),
    "mouse_table": ("mouse_table", mouse_table),
    "position": ("position", position),
    "press_mouse": ("press_mouse", press_mouse),
    "release_mouse": ("release_mouse", release_mouse),
    "scroll": ("scroll", scroll),
    "set_position": ("set_position", set_position),
    "special_table": ("special_table", special_table),
    # keyboard
    "keys_table": ("keys_table", keys_table),
    "type_key": ("type_key", type_key),
    "press_key": ("press_key", press_key),
    "release_key": ("release_key", release_key),
    "check_key_is_press": ("check_key_is_press", check_key_is_press),
    "write": ("write", write),
    "hotkey": ("hotkey", hotkey),
    # image
    "locate_all_image": ("locate_all_image", locate_all_image),
    "locate_image_center": ("locate_image_center", locate_image_center),
    "locate_and_click": ("locate_and_click", locate_and_click),
    # screen
    "size": ("size", size),
    "screenshot": ("screenshot", screenshot)
}


def execute_event(action):
    event = event_dict.get(action[0])
    if event[0] in ["click_mouse"]:
        event[1](action[0], action[1], action[2])
    elif event[0] in ["type_key", "press_key", "release_key", "check_key_is_press", "write"]:
        event[1](action[1])
    elif event[0] in ["position", "record", "stop_record", "size"]:
        event[1]()
    elif event[0] in ["set_position", "screenshot"]:
        event[1](action[1], action[2])
    elif event[0] in ["locate_all_image", "locate_image_center", "press_mouse", "release_mouse"]:
        event[1](action[1], action[2], action[3])
    elif event[0] in ["scroll", "locate_and_click"]:
        event[1](action[1], action[2], action[3], action[4])


def execute_action(action_list: list):
    """
    :param action_list the list include action
    for loop the list and execute action
    """
    execute_record_string = ""
    if action_list is None:
        raise AutoControlActionNullException(action_is_null_error)
    for action in action_list:
        try:
            execute_event(action)
        except AutoControlActionException:
            raise AutoControlActionException(cant_execute_action_error)
        temp_string = "execute: " + str(action)
        print(temp_string)
        execute_record_string = "".join([execute_record_string, temp_string + "\n"])
    return execute_record_string


if __name__ == "__main__":
    import sys

    test_list = None
    if sys.platform in ["win32", "cygwin", "msys"]:
        test_list = [
            ("type_key", 65),
            ("mouse_left", 500, 500),
            ("position", "position"),
            ("press_mouse", "mouse_left", 500, 500),
            ("release_mouse", "mouse_left", 500, 500),
        ]
    elif sys.platform in ["linux", "linux2"]:
        test_list = [
            ("type_key", 38),
            ("mouse_left", 500, 500),
            ("position", "position"),
            ("press_mouse", "mouse_left", 500, 500),
            ("release_mouse", "mouse_left", 500, 500)
        ]
    elif sys.platform in ["darwin"]:
        test_list = [
            ("type_key", 0x00),
            ("mouse_left", 500, 500),
            ("position", "position"),
            ("press_mouse", "mouse_left", 500, 500),
            ("release_mouse", "mouse_left", 500, 500)
        ]
    print("\n\n")
    print(execute_action(test_list))



from je_auto_control.utils.image import template_detection
from je_auto_control.utils.je_auto_control_exception.exception_tag import cant_find_image
from je_auto_control.utils.je_auto_control_exception.exception_tag import find_image_error_variable
from je_auto_control.utils.je_auto_control_exception.exceptions import ImageNotFoundException
from je_auto_control.wrapper.auto_control_mouse import click_mouse
from je_auto_control.wrapper.auto_control_mouse import set_position


def locate_all_image(image, detect_threshold: float = 1, draw_image: bool = False):
    """
    :param image which image we want to find on screen
    :param detect_threshold detect precision 0.0 ~ 1.0; 1 is absolute equal
    :param draw_image draw detect tag on return image
    """
    try:
        image_data_array = template_detection.find_image_multi(image, detect_threshold, draw_image)
    except ImageNotFoundException:
        raise ImageNotFoundException(find_image_error_variable)
    if image_data_array[0] is True:
        return image_data_array[1]
    else:
        raise ImageNotFoundException(cant_find_image)


def locate_image_center(image, detect_threshold: float = 1, draw_image: bool = False):
    """
    :param image which image we want to find on screen
    :param detect_threshold detect precision 0.0 ~ 1.0; 1 is absolute equal
    :param draw_image draw detect tag on return image
    """
    try:
        image_data_array = template_detection.find_image(image, detect_threshold, draw_image)
    except ImageNotFoundException:
        raise ImageNotFoundException(find_image_error_variable)
    if image_data_array[0] is True:
        height = image_data_array[1][2] - image_data_array[1][0]
        width = image_data_array[1][3] - image_data_array[1][1]
        center = [int(height / 2), int(width / 2)]
        return [image_data_array[1][0] + center[0], image_data_array[1][1] + center[1]]
    else:
        raise ImageNotFoundException(cant_find_image)


def locate_and_click(image, mouse_keycode: int, detect_threshold: float = 1, draw_image: bool = False):
    """
    :param image which image we want to find on screen
    :param mouse_keycode which mouse keycode we want to click
    :param detect_threshold detect precision 0.0 ~ 1.0; 1 is absolute equal
    :param draw_image draw detect tag on return image
    """
    try:
        image_data_array = template_detection.find_image(image, detect_threshold, draw_image)
    except ImageNotFoundException:
        raise ImageNotFoundException(find_image_error_variable)
    if image_data_array[0] is True:
        height = image_data_array[1][2] - image_data_array[1][0]
        width = image_data_array[1][3] - image_data_array[1][1]
        center = [int(height / 2), int(width / 2)]
        image_center_x = image_data_array[1][0] + center[0]
        image_center_y = image_data_array[1][1] + center[1]
        set_position(int(image_center_x), int(image_center_y))
        click_mouse(mouse_keycode)
        return [image_center_x, image_center_y]
    else:
        raise ImageNotFoundException(cant_find_image)



"""
general
"""


class AutoControlException(Exception):
    pass


"""
Keyboard
"""


class AutoControlKeyboardException(AutoControlException):
    pass


class AutoControlCantFindKeyException(AutoControlException):
    pass


"""
Mouse
"""


class AutoControlMouseException(AutoControlException):
    pass


"""
Screen
"""


class AutoControlScreenException(AutoControlException):
    pass


"""
Image detect
"""


class ImageNotFoundException(AutoControlException):
    pass


"""
Record
"""


class AutoControlRecordException(AutoControlException):
    pass


"""
Execute action 
"""


class AutoControlJsonActionException(AutoControlException):
    pass


class AutoControlActionNullException(AutoControlException):
    pass


class AutoControlActionException(AutoControlException):
    pass



import sys

from je_auto_control.utils.je_auto_control_exception.exception_tag import osx_import_error
from je_auto_control.utils.je_auto_control_exception.exceptions import AutoControlException

if sys.platform not in ["darwin"]:
    raise AutoControlException(osx_import_error)

import time

import AppKit
import Quartz

from je_auto_control.osx.core.utils.osx_vk import osx_key_shift

special_key_table = {
    "key_sound_up": 0,
    "key_sound_down": 1,
    "key_brightness_up": 2,
    "key_brightness_down": 3,
    "key_capslock": 4,
    "key_help": 5,
    "key_power": 6,
    "key_mute": 7,
    "key_arrow_up": 8,
    "key_arrow_down": 9,
    "key_numlock": 10,
    "key_contrast_up": 11,
    "key_contrast_down": 12,
    "key_launch_panel": 13,
    "key_eject": 14,
    "key_vidmirror": 15,
    "key_play": 16,
    "key_next": 17,
    "key_previous": 18,
    "key_fast": 19,
    "key_rewind": 20,
    "key_illumination_up": 21,
    "key_illumination_down": 22,
    "key_illumination_toggle": 23,
}


def normal_key(keycode: int, is_shift: bool, is_down: bool):
    """
    :param keycode which keycode we want to press or release
    :param is_shift use shift key ?
    :param is_down is_down true = press; false = release
    create event
    post event
    """
    if is_shift:
        event = Quartz.CGEventCreateKeyboardEvent(
            None,
            osx_key_shift,
            is_down
        )
        Quartz.CGEventPost(Quartz.kCGHIDEventTap, event)
    event = Quartz.CGEventCreateKeyboardEvent(
        None,
        keycode,
        is_down
    )
    Quartz.CGEventPost(Quartz.kCGHIDEventTap, event)


def special_key(keycode: int, is_shift: bool):
    """
    :param keycode which keycode we want to press or release
    :param is_shift use shift key ?
    create event
    post event
    """
    keycode = special_key_table[keycode]
    event = AppKit.NSEvent.otherEventWithType_location_modifierFlags_timestamp_windowNumber_context_subtype_data1_data2(
        Quartz.NSSystemDefined,
        (0, 0),
        0xa00 if is_shift else 0xb00,
        0,
        0,
        0,
        8,
        (keycode << 16) | ((0xa if is_shift else 0xb) << 8),
        -1
    )
    Quartz.CGEventPost(0, event)


def press_key(keycode: int, is_shift: bool):
    """
    :param keycode which keycode we want to press
    :param is_shift is shift press?
    """
    if keycode in special_key_table:
        special_key(keycode, is_shift)
    else:
        normal_key(keycode, is_shift, True)


def release_key(keycode: int, is_shift: bool):
    """
    :param keycode which keycode we want to release
    :param is_shift is shift press?
    """
    if keycode in special_key_table:
        special_key(keycode, is_shift)
    else:
        normal_key(keycode, is_shift, False)



import sys

from je_auto_control.utils.je_auto_control_exception.exception_tag import mouse_click_mouse
from je_auto_control.utils.je_auto_control_exception.exception_tag import mouse_press_mouse
from je_auto_control.utils.je_auto_control_exception.exception_tag import mouse_release_mouse
from je_auto_control.utils.je_auto_control_exception.exception_tag import mouse_set_position
from je_auto_control.utils.je_auto_control_exception.exceptions import AutoControlCantFindKeyException
from je_auto_control.utils.je_auto_control_exception.exceptions import AutoControlMouseException
from je_auto_control.wrapper.auto_control_screen import size
from je_auto_control.wrapper.platform_wrapper import mouse
from je_auto_control.wrapper.platform_wrapper import mouse_table
from je_auto_control.wrapper.platform_wrapper import special_table


def position():
    """
    get mouse current position
    """
    try:
        return mouse.position()
    except Exception:
        raise AutoControlMouseException(mouse_get_position)


def set_position(x: int, y: int):
    """
    :param x set position x
    :param y set position y
    """
    try:
        mouse.set_position(x=x, y=y)
    except Exception:
        raise AutoControlMouseException(mouse_set_position)


def press_mouse(mouse_keycode: int, x: int = None, y: int = None):
    """
    :param mouse_keycode which mouse keycode we want to press
    :param x event x
    :param y event y
    """
    try:
        mouse_keycode = mouse_table.get(mouse_keycode)
    except Exception:
        raise AutoControlCantFindKeyException(table_cant_find_key)
    try:
        now_x, now_y = position()
        if x is None:
            x = now_x
        if y is None:
            y = now_y
        if sys.platform in ["win32", "cygwin", "msys", "linux", "linux2"]:
            mouse.press_mouse(mouse_keycode)
        elif sys.platform in ["darwin"]:
            mouse.press_mouse(x, y, mouse_keycode)
    except Exception:
        raise AutoControlMouseException(mouse_press_mouse)


def release_mouse(mouse_keycode: int, x: int = None, y: int = None):
    """
    :param mouse_keycode which mouse keycode we want to release
    :param x event x
    :param y event y
    """
    try:
        mouse_keycode = mouse_table.get(mouse_keycode)
    except Exception:
        raise AutoControlCantFindKeyException(table_cant_find_key)
    try:
        now_x, now_y = position()
        if x is None:
            x = now_x
        if y is None:
            y = now_y
    except Exception:
        raise AutoControlMouseException(mouse_get_position)
    try:
        if sys.platform in ["win32", "cygwin", "msys", "linux", "linux2"]:
            mouse.release_mouse(mouse_keycode)
        elif sys.platform in ["darwin"]:
            mouse.release_mouse(x, y, mouse_keycode)
    except Exception:
        raise AutoControlMouseException(mouse_release_mouse)


def click_mouse(mouse_keycode: int, x: int = None, y: int = None):
    """
    :param mouse_keycode which mouse keycode we want to click
    :param x event x
    :param y event y
    """
    try:
        mouse_keycode = mouse_table.get(mouse_keycode)
    except Exception:
        raise AutoControlCantFindKeyException(table_cant_find_key)
    try:
        now_x, now_y = position()
        if x is None:
            x = now_x
        if y is None:
            y = now_y
    except Exception:
        raise AutoControlMouseException(mouse_get_position)
    try:
        mouse.click_mouse(mouse_keycode, x, y)
    except Exception:
        raise AutoControlMouseException(mouse_click_mouse)


def scroll(scroll_value: int, x: int = None, y: int = None, scroll_direction: str = "scroll_down"):
    """"
    :param scroll_value scroll count
    :param x event x
    :param y event y
    :param scroll_direction which direction we want
    scroll_direction = scroll_up : direction up
    scroll_direction = scroll_down : direction down
    scroll_direction = scroll_left : direction left
    scroll_direction = scroll_right : direction right
    """
    try:
        now_cursor_x, now_cursor_y = position()
    except Exception:
        raise AutoControlMouseException(mouse_get_position)
    width, height = size()
    if x is None:
        x = now_cursor_x
    else:
        if x < 0:
            x = 0
        elif x >= width:
            x = width - 1
    if y is None:
        y = now_cursor_y
    else:
        if y < 0:
            y = 0
        elif y >= height:
            y = height - 1
    try:
        if sys.platform in ["win32", "cygwin", "msys"]:
            mouse.scroll(scroll_value, x, y)
        elif sys.platform in ["darwin"]:
            mouse.scroll(scroll_value)
        elif sys.platform in ["linux", "linux2"]:
            scroll_direction = special_table.get(scroll_direction)
            mouse.scroll(scroll_value, scroll_direction)
    except Exception:
        raise AutoControlMouseException(mouse_click_mouse)



import sys

from je_auto_control.utils.je_auto_control_exception.exceptions import AutoControlException
from je_auto_control.utils.je_auto_control_exception.exception_tag import linux_import_error

if sys.platform not in ["linux", "linux2"]:
    raise AutoControlException(linux_import_error)

import os
from Xlib.display import Display
"""
get x system display
"""
display = Display(os.environ['DISPLAY'])



from PIL import ImageGrab

from je_auto_control.utils.je_auto_control_exception import exception_tag
from je_auto_control.utils.je_auto_control_exception.exceptions import AutoControlScreenException


def pil_screenshot(file_path: str = None, region: list = None):
    """
    :param file_path save screenshot path
    :param region screenshot region
    """
    if region is not None:
        image = ImageGrab.grab(bbox=region)
    else:
        image = ImageGrab.grab()
    if file_path is not None:
        image.save(file_path)
    return image



import sys


from je_auto_control.utils.je_auto_control_exception.exception_tag import osx_import_error
from je_auto_control.utils.je_auto_control_exception.exceptions import AutoControlException

if sys.platform not in ["darwin"]:
    raise AutoControlException(osx_import_error)

from je_auto_control.osx.listener.osx_listener import osx_record
from je_auto_control.osx.listener.osx_listener import osx_stop_record

from je_auto_control.utils.je_auto_control_exception.exceptions import AutoControlJsonActionException


class OSXRecorder(object):

    def record(self):
        osx_record()

    def stop_record(self):
        record_queue = osx_stop_record()
        if record_queue is None:
            raise AutoControlJsonActionException
        return osx_stop_record()


osx_recorder = OSXRecorder()

if __name__ == "__main__":
    test_osx_recorder = OSXRecorder()
    test_osx_recorder.record()
    temp = test_osx_recorder.stop_record()
    print(temp)
    for action in temp.queue:
        print(action)
    while True:
        pass



import sys

from je_auto_control.utils.je_auto_control_exception.exception_tag import windows_import_error
from je_auto_control.utils.je_auto_control_exception.exceptions import AutoControlException

if sys.platform not in ["win32", "cygwin", "msys"]:
    raise AutoControlException(windows_import_error)

from je_auto_control.windows.listener.win32_keyboard_listener import Win32KeyboardListener
from je_auto_control.windows.listener.win32_mouse_listener import Win32MouseListener

from queue import Queue


class Win32Recorder(object):

    def __init__(self):
        self.mouse_record_listener = None
        self.keyboard_record_listener = None
        self.record_queue = None
        self.result_queue = None

    def record(self):
        self.mouse_record_listener = Win32MouseListener()
        self.keyboard_record_listener = Win32KeyboardListener()
        self.record_queue = Queue()
        self.mouse_record_listener.record(self.record_queue)
        self.keyboard_record_listener.record(self.record_queue)

    def stop_record(self):
        self.result_queue = self.mouse_record_listener.stop_record()
        self.result_queue = self.keyboard_record_listener.stop_record()
        self.record_queue = None
        return self.result_queue

    def record_mouse(self):
        self.mouse_record_listener = Win32MouseListener()
        self.record_queue = Queue()
        self.mouse_record_listener.record(self.record_queue)

    def stop_record_mouse(self):
        self.result_queue = self.mouse_record_listener.stop_record()
        self.record_queue = None
        return self.result_queue

    def record_keyboard(self):
        self.keyboard_record_listener = Win32KeyboardListener()
        self.record_queue = Queue()
        self.keyboard_record_listener.record(record_queue)

    def stop_record_keyboard(self):
        self.result_queue = self.keyboard_record_listener.stop_record()
        self.record_queue = None
        return self.result_queue


win32_recorder = Win32Recorder()

if __name__ == "__main__":
    win32_recorder = Win32Recorder()
    win32_recorder.record()
    from time import sleep

    sleep(10)
    for i in win32_recorder.stop_record().queue:
        print(i)



import setuptools

with open("README.md", "r") as README:
    long_description = README.read()

setuptools.setup(
    name="je_auto_control",
    version="0.0.66",
    author="JE-Chen",
    author_email="zenmailman@gmail.com",
    description="auto testing",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/JE-Chen/AutoControl",
    packages=setuptools.find_packages(),
    install_requires=[
        "je_open_cv",
        "pillow",
        "numpy",
        "pyobjc-core;platform_system=='Darwin'",
        "pyobjc;platform_system=='Darwin'",
        "python3-Xlib;platform_system=='Linux'"
    ],
    classifiers=[
        "Programming Language :: Python :: 3.5",
        "Development Status :: 2 - Pre-Alpha",
        "Environment :: Win32 (MS Windows)",
        "Environment :: MacOS X",
        "Environment :: X11 Applications",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent"
    ]
)

# python setup.py sdist bdist_wheel
# python -m twine upload dist/*
